<?php
/**
 * Plugin Name: Cherry Checkout
 * Plugin URI: https://developers.cherrycheckout.com/en/plugins/woocommerce
 * Description: Offrez à vos clients la possibilité d'ajouter la cerise sur le gateau !
 * Version: 2.1.1
 * Author: Cherry Checkout SA
 * Author URI: https://www.cherrycheckout.com
 */

if ( ! defined( 'ABSPATH' ) ) exit;

if ( ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) exit;

define( 'CHERRYCHECKOUT_DIR', plugin_dir_path( __FILE__ ) );
define( 'CHERRYCHECKOUT_URL', plugin_dir_url( __FILE__ ) );
define( 'CHERRYCHECKOUT_PLUGIN_ID', 'cherrycheckout');

require_once( CHERRYCHECKOUT_DIR . 'class/CherryCheckoutApi.php' );
require_once( CHERRYCHECKOUT_DIR . 'class/CherryCheckoutLogger.php' );
require_once( CHERRYCHECKOUT_DIR . 'class/CherryCheckoutWordpressConfig.php' );

function _initApi() {
	$settings = unserialize( get_option( CHERRYCHECKOUT_PLUGIN_ID . '_api_settings' ) );

	if ( ! $settings ) {
		throw new \Exception('Cherry checkout settings missing');
	}

	$cherrycheckoutApi = new CherryCheckoutApi( $settings['api'], $settings['publicKey'], $settings['privateKey'] );
	return $cherrycheckoutApi;
}

function _getMe() {

	$meInCacheDuration = 60;

	// Get me from config
	$me = unserialize( get_option( CHERRYCHECKOUT_PLUGIN_ID . '_me' ) );

	// Check how the me is old
	if ($me) {
		$old = date_diff(new DateTime(), $me->configurationUpdatedAt);
		$totalMinutes = (($old->y * 365.25 + $old->m * 30 + $old->d) * 24 + $old->h) * 60 + $old->i + $old->s/60;

		// Set me as obsolete (or not)
		$me->isObsolete = !!($totalMinutes > $meInCacheDuration);
	}

	if (!$me || $me->isObsolete) {

		try {
			// Reset me
			$me = _resetMe();
		}
		catch (\Exception $ex) {
			throw $ex;
		}
	}

	return $me;
}

function _resetMe() {
	try {
		$cherrycheckoutApi = _initApi();

		// Get from api
		$me = $cherrycheckoutApi->me( true );

		if ( ! $me ) {
			throw new \Exception( 'Me is undefined' );
		}

		// Stock me in Configuration
		$me->configurationUpdatedAt = new DateTime();
		update_option( CHERRYCHECKOUT_PLUGIN_ID . '_me', serialize( $me ) );

		// Rename cherry products depending of contract (donation/game parts)
		_renameCherryProduct( $me->contract->charityOrderPercentage, $me->contract->gameOrderPercentage );

		return $me;
	}
	catch (\Exception $ex) {
		throw $ex;
	}
}

function _renameCherryProduct($charityPart, $gamePart) {

	$productSettings = unserialize( get_option ( CHERRYCHECKOUT_PLUGIN_ID . '_product_settings' ) );

	$language = substr(get_locale(), 0, 2);
	$productName = 'Cherry Checkout';

	if ($charityPart > 0 && $gamePart > 0) {
		switch ($language) {
			case 'fr':
				$productName = 'Cherry Checkout - Don et Concours';
				break;
			default:
				$productName = 'Cherry Checkout - Donation and Contest';
				break;
		}
	}
	else if ($charityPart > 0 && $gamePart <= 0) {
		switch ($language) {
			case 'fr':
				$productName = 'Cherry Checkout - Don';
				break;
			default:
				$productName = 'Cherry Checkout - Donation';
				break;
		}
	}
	else if ($charityPart <= 0 && $gamePart > 0) {
		switch ($language) {
			case 'fr':
				$productName = 'Cherry Checkout - Concours';
				break;
			default:
				$productName = 'Cherry Checkout - Contest';
				break;
		}
	}

	$product = array(
		'ID'				=> $productSettings['productId'],
		'post_name'			=> $productName,
		'post_title'		=> $productName,
	);

	wp_update_post($product);
}

function _getCartAmountWithoutCherries()
{
	$products = WC()->cart->get_cart();
	$productSettings = unserialize( get_option( CHERRYCHECKOUT_PLUGIN_ID . '_product_settings' ) );

	$cartAmount = WC()->cart->cart_contents_total;

	// Subtract cherries
	foreach ($products as $product) {
		if ($product['product_id'] == $productSettings['productId']) {
			$cartAmount -= (float)$product['line_total'];
		}
	}

	return (float)$cartAmount;
}

function _getMinimalCherriesRequiredAccordingToCart($priceScale)
{
	$cartAmount = _getCartAmountWithoutCherries();

	foreach ($priceScale as $ps) {
		if (($cartAmount > (float)($ps->from)) && $cartAmount <= (float)($ps->to)) {
			return (int)$ps->amount;
		}
	}

	return (int)$priceScale[0]->amount;
}

function _getPluginContent($plugin, $minimalCherries)
{
	$locale = substr( get_locale(), 0, 2 );

	// Get the plugin in the customer language if possible
	$plugin = (array)$plugin;
	$plugin = ( isset($plugin[ $locale ] ) ) ? $plugin[ $locale ] : $plugin[ 'en' ];

	$currency = get_woocommerce_currency();
	if ($currency == 'EUR') {
		$currency = '€';
	}

	// Insert the good price in the cherrycheckout form button
	$plugin = str_replace('{currency}', $currency, $plugin);
	$plugin = str_replace('{cherryPrice}', (int)($minimalCherries), $plugin);

	return $plugin;
}

function _getCherryFromCart() {

	$products = WC()->cart->get_cart();
	$productSettings = unserialize( get_option( CHERRYCHECKOUT_PLUGIN_ID . '_product_settings' ) );

	foreach ($products as $cartItemKey => $product) {
		if ($product['product_id'] == $productSettings['productId']) {
			$product['cartItemKey'] = $cartItemKey;
			return $product;
		}
	}

	return false;
}

function _getCherryFromOrder($order) {

	$products = $order->get_items();
	$productSettings = unserialize( get_option( CHERRYCHECKOUT_PLUGIN_ID . '_product_settings' ) );

	foreach ($products as $cartItemKey => $product) {
		if ($product['product_id'] == $productSettings['productId']) {
			$product['cartItemKey'] = $cartItemKey;
			return $product;
		}
	}

	return false;
}

function _getApiKeys() {

	$settings = unserialize( get_option( CHERRYCHECKOUT_PLUGIN_ID . '_api_settings' ) );
	return array(
		'publicKey' => $settings['publicKey'],
		'privateKey' => $settings['privateKey']
	);
}

/**
 * HOOKS
 */
function cherrycheckout_activation() {

	// Create the product
	$product = array(
		'post_content'		=> '',
		'post_name'			=> 'Cherry Checkout',
		'post_title'		=> 'Cherry Checkout',
		'post_status'		=> 'private',
		'post_type'			=> 'product',
		'post_author'		=> 1,
		'ping_status'		=> 'closed',
		'guid'				=> 'cherrycheckout',
		'post_excerpt'		=> '',
		'post_date'			=> date('Y-m-d H:i:s'),
		'post_date_gmt'		=> date('Y-m-d H:i:s'),
		'comment_status'	=> 'closed'
	);

	$product_id = wp_insert_post( $product, true );

	if (! $product_id ) {
		echo 'Unable to create product';
		return false;
	}

	// Set product in a category
	wp_set_object_terms($product_id, 'Cherry Checkout', 'product_cat');
	wp_set_object_terms($product_id, 'simple', 'product_type');

	update_post_meta($product_id, '_tax_status', 'none');
	update_post_meta($product_id, '_visibility', 'hidden');
	update_post_meta($product_id, '_stock_status', 'instock');
	update_post_meta($product_id, '_regular_price', 1 );
	update_post_meta($product_id, '_price', 1 );
	update_post_meta($product_id, '_langue', 'fr');

	// Save product image
	$thumb_id = null;
	try {
		$tmp = download_url( CHERRYCHECKOUT_URL.'assets/img/cherryProductImage.png' );

		if ( ! is_wp_error( $tmp ) ) {
			$file_array = array(
				'name'		=> 'cherryProductImage.png',
				'tmp_name'	=> $tmp,
			);

			$thumb_id = media_handle_sideload( $file_array, $product_id );

			if( $thumb_id ) {
				add_post_meta( $product_id, '_thumbnail_id', $thumb_id );
			}
			else {
				throw new \Exception('No $thumb_id');
			}
		}
		else {
			throw new \Exception('Unable to get image : ' . CHERRYCHECKOUT_URL.'assets/img/cherryProductImage.png');
		}
	}
	catch (\Exception $ex) {
		CherryCheckoutLogger::add('[cherrycheckout_activation] ERROR : ' . $ex->getMessage(), 300011);
	}

	$productSettings = array(
		'productId'			=> $product_id,
		'productImageId'	=> $thumb_id,
	);

	// Save product settings in config
	update_option( CHERRYCHECKOUT_PLUGIN_ID . '_product_settings', serialize($productSettings) );

	// If there is already an api settings, send plugin activation info to API
	try {
		$cherrycheckoutApi = _initApi();

		if ( $cherrycheckoutApi ) {
			$values = CherryCheckoutWordpressConfig::get();
			$values['pluginInstalled'] = true;
			$cherrycheckoutApi->updateMe( $values );
		}
	}
	catch ( \Exception $ex ) {
		CherryCheckoutLogger::add('[cherrycheckout_activation] ERROR : ' . $ex->getMessage(), 300012);
	}

	return true;
}

function cherrycheckout_deactivation() {

	// Remove the cherry checkout product
	$productSettings = unserialize( get_option ( CHERRYCHECKOUT_PLUGIN_ID . '_product_settings' ) );

	wp_delete_post($productSettings['productId']);
	wp_delete_post($productSettings['productImageId']);

	try {
		$cherrycheckoutApi = _initApi();

		$values = CherryCheckoutWordpressConfig::get();
		$values['pluginInstalled'] = false;
		$cherrycheckoutApi->updateMe( $values );
	}
	catch ( \Exception $ex ) {
		CherryCheckoutLogger::add('[cherrycheckout_deactivation] ERROR : ' . $ex->getMessage(), 300020);
	}

	return true;
}

function cherrycheckout_uninstall() {

	try {
		$cherrycheckoutApi = _initApi();

		$values = CherryCheckoutWordpressConfig::get();
		$values['pluginInstalled'] = false;
		$cherrycheckoutApi->updateMe( $values );
	}
	catch ( \Exception $ex ) {
		CherryCheckoutLogger::add('[cherrycheckout_uninstall] ERROR : ' . $ex->getMessage(), 300030);
	}
}

function cherrycheckout_load_translations() {
	load_plugin_textdomain( CHERRYCHECKOUT_PLUGIN_ID, false, basename( dirname( __FILE__ ) ) . '/lang' );
}

function cherrycheckout_add_settings_tab($settings_tabs) {
	$settings_tabs[ CHERRYCHECKOUT_PLUGIN_ID ] = 'Cherry Checkout';
	return $settings_tabs;
}


function cherrycheckout_show_settings() {
	include( 'includes/show-settings.php' );
}

function cherrycheckout_save_settings() {
	include( 'includes/save-settings.php' );
}

function cherrycheckout_plugin_scripts() {
	include ( 'includes/plugin-scripts.php' );
}

function cherrycheckout_display_plugin() {
	include( 'includes/display-plugin.php' );
}

function cherrycheckout_load_cart() {
	include( 'includes/load-cart.php' );
}

function cherrycheckout_create_order($order_id) {
	include( 'includes/create-order.php' );
}

function cherrycheckout_update_order($order_id) {
	include( 'includes/update-order.php' );
}

register_activation_hook( __FILE__, 'cherrycheckout_activation' );
register_deactivation_hook( __FILE__, 'cherrycheckout_deactivation' );
register_uninstall_hook( __FILE__, 'cherrychekout_uninstall' );

add_action( 'plugins_loaded', 'cherrycheckout_load_translations');

add_filter( 'woocommerce_settings_tabs_array', 'cherrycheckout_add_settings_tab', 50 );
add_action( 'woocommerce_settings_tabs_' . CHERRYCHECKOUT_PLUGIN_ID, 'cherrycheckout_show_settings' );
add_action( 'woocommerce_settings_save_' . CHERRYCHECKOUT_PLUGIN_ID, 'cherrycheckout_save_settings' );

add_action( 'wp_footer', 'cherrycheckout_plugin_scripts' );
add_action( 'woocommerce_after_cart_table', 'cherrycheckout_display_plugin' );
add_action( 'woocommerce_cart_loaded_from_session', 'cherrycheckout_load_cart' );
add_action( 'woocommerce_thankyou', 'cherrycheckout_create_order' );
add_action( 'woocommerce_order_status_changed', 'cherrycheckout_update_order' );
